@@ -2,7 +2,7 @@
 
 #
 # dbcolscorrelate.pm
-# Copyright (C) 1998-2022 by John Heidemann <johnh@isi.edu>
+# Copyright (C) 1998-2024 by John Heidemann <johnh@isi.edu>
 #
 # This program is distributed under terms of the GNU general
 # public license, version 2.  See the file COPYING
@@ -21,7 +21,7 @@ dbcolscorrelate - find the coefficient of correlation over columns
 
 =head1 DESCRIPTION
 
-Compute the coefficient of correlation over two (or more) columns.
+Compute the Pearson coefficient of correlation over two (or more) columns.
 
 The output is one line of correlations.
 
@@ -48,13 +48,17 @@ This program requires a complete copy of the input data on disk.
 
 =item B<--sample>
 
-Select a the Pearson product-moment correlation coefficient
+Select a the sample Pearson product-moment correlation coefficient
 (the "sample correlation coefficient", usually designated I<r>).
 
-=item B<--nosample>
+=item B<--no-sample>
 
 Select a the Pearson product-moment correlation coefficient
-(the "sample correlation coefficient", usually designated I<r>).
+(the "correlation coefficient", usually designated E<0x03c1>).
+
+=item B<--weight> COL
+
+Weight the correlation by column COL.
 
 
 
@@ -118,23 +122,28 @@ end_standard_fsdb_options
 
 =head2 Input:
 
-    #fsdb name id test1 test2
-    a 1 80 81
-    b 2 70 71
-    c 3 65 66
-    d 4 90 91
-    e 5 70 71
-    f 6 90 91
+    #fsdb i_x i_y
+    10.0   8.04
+    8.0    6.95
+    13.0   7.58
+    9.0    8.81
+    11.0   8.33
+    14.0   9.96
+    6.0    7.24
+    4.0    4.26
+    12.0  10.84
+    7.0    4.82
+    5.0    5.68
 
 =head2 Command:
 
-    cat DATA/more_grades.fsdb | dbcolscorrelate test1 test2
+    cat TEST/anscombe_quartet.in | dbcolscorrelate i_x i_y
 
 =head2 Output:
 
     #fsdb correlation:d
-    0.83329
-    #  | dbcolscorrelate test1 test2
+    0.81642
+    #  | dbcolscorrelate i_x i_y
 
 
 =head1 SEE ALSO
@@ -199,6 +208,7 @@ sub set_defaults ($) {
     $self->{_include_non_numeric} = undef;
     $self->{_sample} = undef;
     $self->{_fscode} = undef;
+    $self->{_weight_column} = undef;
     $self->set_default_tmpdir;
 }
 
@@ -228,6 +238,7 @@ sub parse_options ($@) {
 	's|sample!' =>  \$self->{_sample},
 	'T|tmpdir|tempdir=s' => \$self->{_tmpdir},
 	'o|output=s' => sub { $self->parse_io_option('output', @_); },
+	'w|weight=s' => \$self->{_weight_column},
 	) or pod2usage(2);
     push (@{$self->{_columns}}, @argv);
 }
@@ -243,7 +254,15 @@ Internal: setup, parse headers.
 sub setup ($) {
     my($self) = @_;
 
-    $self->finish_io_option('input', -comment_handler => $self->create_pass_comments_sub);
+    $self->finish_io_option('input', -comment_handler => $self->create_delay_comments_sub);
+
+    if (defined($self->{_weight_column})) {
+        $self->{_weight_coli} = $self->{_in}->col_to_i($self->{_weight_column});
+	croak($self->{_prog} . ": weight " . $self->{_weight_column} . " does not exist in the input stream.\n")
+	    if (!defined($self->{_weight_coli}));
+    } else {
+        $self->{_weight_coli} = undef;
+    };
 
     croak($self->{_prog} . ": at least two columns must be specified to compute a correlation.\n")
 	if ($#{$self->{_columns}} < 1);
@@ -299,36 +318,76 @@ sub run ($) {
     my(@stats_threads);
     my $columns_aref = $self->{_columns};
     my $colis_aref = $self->{_colis};
+    my(@stats_cols) = qw(data);
+    my(@stats_args) = (($self->{_sample} ? '--sample' : '--nosample'));
+    if (defined($self->{_weight_coli})) {
+        push(@stats_cols, 'weight');
+        push(@stats_args, '--weight' => 'weight');
+    };
+    push(@stats_args, 'data');
     foreach (0..$#$columns_aref) {
 	($stats_source_queues[$_], $stats_sinks[$_], $stats_threads[$_]) =
-	    dbpipeline_open2([-cols => [qw(data)]], dbcolstats(($self->{_sample} ? '--sample' : '--nosample'), 'data'));
+	    dbpipeline_open2([-cols => \@stats_cols], dbcolstats(@stats_args));
     }
     my $fref;
-    while ($fref = &$read_fastpath_sub()) {
-	# copy and send to stats
-	$copy_writer->write_rowobj($fref);
-	# with forking we have to close output explicitly
-	# otherwise we block on the pipe(2) to the subprocesses.
-	foreach (0..$#$columns_aref) {
-	    $stats_sinks[$_]->write_row($fref->[$colis_aref->[$_]]);
-	};
+    my $loop_code = q'
+        while ($fref = &$read_fastpath_sub()) {
+            # copy and send to stats
+	    $copy_writer->write_rowobj($fref);
+	    # with forking we have to close output explicitly
+	    # otherwise we block on the pipe(2) to the subprocesses.
+	    foreach (0..$#$columns_aref) {
+                $stats_sinks[$_]->write_row($fref->[$colis_aref->[$_]]';
+    if (defined($self->{_weight_coli})) {
+        $loop_code .= ', $fref->[' . $self->{_weight_coli} . ']';
     };
+    $loop_code .= q');
+	    };
+        };
+    ';
+    eval $loop_code;
     # close up both
     $copy_writer->close;
     foreach (0..$#$columns_aref) {
 	$stats_sinks[$_]->close;
 	$stats_sinks[$_] = undef;
     };
+
+    #
+    # The definition of correlation coefficient:
+    #
+    # corr(X,Y) = 
+    # \rho_{X,Y} = cov(X,Y) / (\sigma_X \sigma_Y)
+    #
+    # where cov(X,Y) = E[ (X - \mu_X) (Y - \mu_Y) ]
+    #
+    # To add weighting:
+    # corr(x,y;w) = cov(x,y;w) / \sqrt( cov(x,x;w) cov(y,y;w) )
+    #    or corr(x,y,w) = cov(x, y, w) / np.sqrt(cov(x, x, w) * cov(y, y, w))
+    # where cov(X,Y;w) = \Sigma_i ( w_i ( x_i - m(x;w)) ( y_i - m(y;w)))
+    #    or cov(x, y, w) = np.sum(w * (x - m(x, w)) * (y - m(y, w))) / np.sum(w)
+    # where m(x;y) = (\Sigma_i (w_i x_i)) / \Sigma_i w_i))
+    #    or m(x,w) = np.sum(x * y) / np.sum(w)
+    #
     my @means;
     my @stddevs;
+    my @ns;
     foreach (0..$#$columns_aref) {
 	my $stats_href = dbpipeline_close2_hash($stats_source_queues[$_], $stats_sinks[$_], $stats_threads[$_]);
-	$means[$_] = $stats_href->{'mean'};
-	croak($self->{_prog} . ": column " . $columns_aref->[$_] . " does not have valid mean.\n")
-	    if (!defined($means[$_]));
-	$stddevs[$_] = $stats_href->{'stddev'};
-	croak($self->{_prog} . ": column " . $columns_aref->[$_] . " does not have valid standard deviation.\n")
-	    if (!defined($stddevs[$_]));
+        for my $stat (qw(mean stddev n)) {
+            my $val = $stats_href->{$stat};
+            croak($self->{_prog} . ": column " . $columns_aref->[$_] . " does not have valid $stat.\n")
+                if (!defined($val));
+            if ($stat eq 'mean') {
+                $means[$_] = $val;
+            } elsif ($stat eq 'stddev') {
+                $stddevs[$_] = $val;
+            } elsif ($stat eq 'n') {
+                $ns[$_] = $val;
+            } else {
+                croak("dbcolscorrelate.pm: interal error bad $stat");
+            };
+        };
     };
 
     #
@@ -339,52 +398,68 @@ sub run ($) {
     $self->{_in}->close;
     $self->{_in} = new Fsdb::IO::Reader(-file => $self->{_copy_filename},
 		    -comment_handler => $self->create_pass_comments_sub);
-    my $sum_zs;
-    my $ns;
+    my $covariance_sums;
+    my $covariances;
     foreach my $i (0..$#$columns_aref) {
 	foreach my $j (0..$#$columns_aref) {
-	    next if ($i >= $j);
-	    $sum_zs->[$i][$j] = 0;
-	    $ns->[$i][$j] = 0;
+	    next if ($i > $j);
+	    $covariance_sums->[$i][$j] = $covariances->[$i][$j] = 0;
 	};
     };
 	    
     $read_fastpath_sub = $self->{_in}->fastpath_sub(); # regenerate with copy stream
-    while ($fref = &$read_fastpath_sub()) {
-	my @zs;
+    my $w_coli = $self->{_weight_coli};
 
-	# compute z-score
+    #
+    # now compute the weighted covariances
+    # cov(X,Y) = E[ (X-E[X,w])(Y-E[Y,w]) ]
+    #
+    while ($fref = &$read_fastpath_sub()) {
+	my @deviations;
+	# compute devations (like z-scores, but not normalized by stddev)
 	foreach (0..$#$columns_aref) {
 	    my $x = $fref->[$colis_aref->[$_]];
 	    if ($x !~ /$is_numeric_regexp/) {
-		$zs[$_] = undef;
+		$deviations[$_] = undef;
 	    } else {
-		$zs[$_] = ($x - $means[$_]) / $stddevs[$_];
+		$deviations[$_] = $x - $means[$_];
 	    };
         };
 
-	# and compute running stats for correlations
+        # compute the covariance_sums
+        my $w = defined($w_coli) ? $fref->[$w_coli] : 1;
+
 	foreach my $i (0..$#$columns_aref) {
 	    foreach my $j (0..$#$columns_aref) {
-		next if ($i >= $j ||!defined($zs[$i]) || !defined($zs[$j]));
-		$sum_zs->[$i][$j] += $zs[$i] * $zs[$j];
-		($ns->[$i][$j])++;
+		next if ($i > $j ||!defined($deviations[$i]) || !defined($deviations[$j]));
+		$covariance_sums->[$i][$j] += $w * $deviations[$i] * $deviations[$j];
 	    };
 	};
     };
+    # and compute weighted covariances
+    foreach my $i (0..$#$columns_aref) {
+	foreach my $j (0..$#$columns_aref) {
+            next if (!defined($covariance_sums->[$i][$j]));
+            my $min_ns = ($ns[$i] <= $ns[$j] ? $ns[$i] : $ns[$j]);
+            $covariances->[$i][$j] = $covariance_sums->[$i][$j] / (1.0 * $min_ns);
+        };
+    };
+
 
     #
     # Finally output the results.
     #
-    my(@correlations);
+    # compute them
+    my @correlations;
     foreach my $i (0..$#$columns_aref) {
 	foreach my $j (0..$#$columns_aref) {
 	    next if ($i >= $j);
-	    if ($ns->[$i][$j] == 0) {
+	    if (!defined($covariances->[$i][$j])) {
 		push(@correlations, $self->{_empty});
 	    } else {
-		my $c = $sum_zs->[$i][$j] / $ns->[$i][$j];
-		push(@correlations, $self->numeric_formatting($c));
+                my $den = sqrt( $covariances->[$i][$i] * $covariances->[$j][$j] );
+                my $corr = ($den == 0 ? "-" : $covariances->[$i][$j] / $den);
+		push(@correlations, $self->numeric_formatting($corr));
             };
         };
     };
@@ -393,7 +468,7 @@ sub run ($) {
 
 =head1 AUTHOR and COPYRIGHT
 
-Copyright (C) 1998-2022 by John Heidemann <johnh@isi.edu>
+Copyright (C) 1998-2024 by John Heidemann <johnh@isi.edu>
 
 This program is distributed under terms of the GNU general
 public license, version 2.  See the file COPYING
