@@ -2,7 +2,7 @@
 
 #
 # dbcolstats.pm
-# Copyright (C) 1991-2018 by John Heidemann <johnh@isi.edu>
+# Copyright (C) 1991-2024 by John Heidemann <johnh@isi.edu>
 #
 # This program is distributed under terms of the GNU general
 # public license, version 2.  See the file COPYING
@@ -17,7 +17,8 @@ dbcolstats - compute statistics on a fsdb column
 
 =head1 SYNOPSIS
 
-dbcolstats  [-amS] [-c ConfidenceFraction] [-q NumberOfQuantiles] column
+dbcolstats  [-amS] [-c ConfidenceFraction] [-q NumberOfQuantiles]
+	    [-w WeightColumn] column
 
 =head1 DESCRIPTION
 
@@ -26,6 +27,9 @@ Records containing non-numeric data are considered null
 do not contribute to the stats (with the C<-a> option
 they are treated as zeros).
 
+Optionally (with C<-w>), data can be weighted by a second column.
+(In this case, I<n> returns a weighted count of input rows.)
+
 Confidence intervals are a t-test (+/- (t_{a/2})*s/sqrt(n))
 and assume the population takes a normal distribution
 with a small number of samples (< 100).
@@ -87,6 +91,11 @@ assuming I<n-1> degrees of freedom.
 Compute I<whole> population statistics
 (e.g., the population standard devation).
 
+=item B<-w> or B<--weight> WeightColumn
+
+Weight the data by the WeightColumn.
+(By default, all rows have weight 1.)
+
 =item B<-S> or B<--pre-sorted>
 
 Assume data is already sorted.
@@ -105,7 +114,7 @@ Defaults on.
 Specify the field (column) separator as C<S>.
 See L<dbfilealter> for valid field separators.
 
-=item B<-T TmpDir>
+=item B<-T TmpDir> or <B--tmpdir TmpDir>
 
 where to put temporary data.
 Only used if median or quantiles are requested.
@@ -292,6 +301,7 @@ sub set_defaults($) {
     $self->{_fscode} = undef;
     $self->{_max_parallelism} = undef;
     $self->{_key_column} = undef;
+    $self->{_weight_column} = undef;
     $self->{_output_on_no_input} = undef;
     $self->set_default_tmpdir;
 }
@@ -330,6 +340,7 @@ sub parse_options($@) {
 	's|sample!' =>  \$self->{_sample},
 	'S|pre-sorted+' => \$self->{_pre_sorted},
 	'T|tmpdir|tempdir=s' => \$self->{_tmpdir},
+	'w|weight=s' => \$self->{_weight_column},
 	'saveoutput=s' => \$self->{_save_output},
 	) or pod2usage(2);
     $self->parse_target_column(\@argv);
@@ -355,6 +366,13 @@ sub setup($) {
     $self->{_target_coli} = $self->{_in}->col_to_i($self->{_target_column});
     croak($self->{_prog} . ": target column " . $self->{_target_column} . " is not in input stream.\n")
 	if (!defined($self->{_target_coli}));
+    if (defined($self->{_weight_column})) {
+        $self->{_weight_coli} = $self->{_in}->col_to_i($self->{_weight_column});
+        croak($self->{_prog} . ": weight column " . $self->{_weight_column} . " is not in input stream.\n")
+            if (!defined($self->{_weight_coli}));
+        croak($self->{_prog} . ": currently dbcolstats does not support weighted median or quantiles.\n")
+            if (defined($self->{_median}) || defined($self->{_quantile}));
+    };
     $self->{_key_coli} = undef;
     if (defined($self->{_key_column})) {
         $self->{_key_coli} = $self->{_in}->col_to_i($self->{_key_column});
@@ -393,6 +411,8 @@ sub setup($) {
 	    my(@dbsort_args) = qw(-n data);
 	    push (@dbsort_args, '--parallelism', $self->{_max_parallelism})
 		if (defined($self->{_max_parallelism}));
+	    push (@dbsort_args, '--tmpdir', $self->{_tmpdir})
+		if (defined($self->{_tmpdir}));
 	    print STDERR "dbcolstats: doing sorter thread\n" if ($self->{_debug} > 2);
 	    ($save_out, $sorter_fred) = dbpipeline_sink(\@writer_args,
 			'--output' => $self->{_save_out_filename},
@@ -504,9 +524,11 @@ sub run_one_key($) {
 
     # xxx: should eval all this to factor out constants from runtime
     my($xf) = $self->{_target_coli};
+    my($xw) = $self->{_weight_coli};
     my($key_column) = $self->{_key_column};
 
     my($n) = 0;
+    my($abs_n) = 0;
     my($sx) = 0;
     my($sxx) = 0;
     my $min;
@@ -514,10 +536,12 @@ sub run_one_key($) {
     my $key;
     my $last_key = $self->{_holdover_key};
     my $holdover_data = $self->{_holdover_data};
-    $self->{_holdover_key} = $self->{_holdover_data} = undef;
+    my $holdover_weight = $self->{_holdover_weight};
+    $self->{_holdover_key} = $self->{_holdover_data} = $self->{_holdover_weight} = undef;
 
     my $fref;
     my $x;
+    my $w;
 
     {
 	my $save_out = $self->{_save_out};
@@ -527,12 +551,17 @@ sub run_one_key($) {
 	    while (1) {
 		if (defined($holdover_data)) {
 		    $x = $holdover_data;   # and key was set earlier
+                ' .
+		    (defined($self->{_weight_column}) ? q'$w = $holdover_weight;' : '') .
+                q'
 		    $holdover_data = undef;
 		} else {
 		    $fref = &{$read_fastpath_sub}();
 		    last if (!defined($fref));
-		    $x = $fref->[' . $xf . q'];
-		';
+		    $x = $fref->[' . $xf . "];\n";
+        if (defined($self->{_weight_column})) {
+            $code .= '$w = $fref->[' . $xw . "];\n";
+        };
 	if (defined($self->{_key_column})) {
 	    $code .= q'
 		    $key = $fref->[' . $self->{_key_coli} . '];
@@ -549,14 +578,18 @@ sub run_one_key($) {
 		};
 	';
 
-	$code .= 'next if ($x !~ /' . $is_numeric_regexp . "/);\n"
-	    if (!$self->{_include_non_numeric});
+        if (!$self->{_include_non_numeric}) {
+            $code .= 'next if ($x !~ /' . $is_numeric_regexp . "/);\n";
+            $code .= '$w = 1 if ($w !~ /' . $is_numeric_regexp . "/);\n" if (defined($self->{_weight_column}));
+        };
 	$code .= q'
 	    $x += 0.0; # force numeric
-	    $n++;
-	    $sx += $x;
-	    $sxx += $x * $x;
-	';
+	    $n += ' . (defined($self->{_weight_column}) ? q'$w' : '1') . ";\n";
+        if (defined($self->{_weight_column})) {
+            $code .= '$abs_n++;' . "\n";
+        };
+	$code .= q'$sx += $x ' . (defined($self->{_weight_column}) ? q' * $w' : '') . q'; 
+	    $sxx += $x * $x'  . (defined($self->{_weight_column}) ? q' * $w' : '') .  ";\n";
 	$code .= 'print STDERR "dbcolstats: save-out write\n";' . "\n" if ($self->{_debug} > 2);
 
 	if ($self->{_quantile} || $self->{_median}) {
@@ -695,7 +728,7 @@ Internal: run over each row, for one or many keys.
 =cut
 sub run($) {
     my($self) = @_;
-    $self->{_holdover_key} = $self->{_holdove_data} = undef;
+    $self->{_holdover_key} = $self->{_holdove_data} = $self->{_holdove_weight} = undef;
     for (;;) {
 	$self->run_one_key();
 	last if (!defined($self->{_holdover_key}));
@@ -704,7 +737,7 @@ sub run($) {
 
 =head1 AUTHOR and COPYRIGHT
 
-Copyright (C) 1991-2018 by John Heidemann <johnh@isi.edu>
+Copyright (C) 1991-2024 by John Heidemann <johnh@isi.edu>
 
 This program is distributed under terms of the GNU general
 public license, version 2.  See the file COPYING
