@@ -153,6 +153,7 @@ $VERSION = 2.0;
 
 use strict;
 use Pod::Usage;
+use PerlIO::encoding;
 use Carp;
 
 use Fsdb::Filter;
@@ -256,14 +257,14 @@ sub run ($) {
     #
     # Read the form.
     #
-    open(FORM, "<" . $self->{_format_file})
+    open(my $form, "<:encoding(UTF-8)", $self->{_format_file})
 	or croak($self->{_prog} . ": cannot open " . $self->{_format_file} . ".\n");
     my @form = ();
-    while (<FORM>) {
+    while (<$form>) {
 	s/\@/\\\@/g;   # quote @'s
 	push(@form, $_);
     };
-    close FORM;
+    close $form;
 
     croak($self->{_prog} . ": no To: line in form.\n")
 	if (!grep(/^To:/i, @form));
@@ -291,9 +292,10 @@ sub run ($) {
     #
     my $fref;
     my $read_fastpath_sub = $self->{_in}->fastpath_sub();
+    binmode(STDOUT, ":utf8");
     print "#!/bin/sh\n" if ($self->{_mechanism} ne 'mh');
     my $mh_seqno = 1;
-    
+
     while ($fref = &$read_fastpath_sub()) {
 	my $result = eval $code;
 	$@ && croak($self->{_prog} . ": internal eval error ``$@''.\n");
@@ -332,7 +334,14 @@ sub run ($) {
 	if ($self->{_mechanism} eq 'sendmail') {
 	    print "sendmail '" . $fields{"to"} . "' <<'$end_of_form_marker'\n$result\n$end_of_form_marker\n\n";
 	} elsif ($self->{_mechanism} eq 'Mail') {
-	    my $cc_arg = (defined($fields{"cc"}) ? "-c '" . $fields{"cc"} . "' " : "");
+	    my $cc_arg = '';
+            if (defined($fields{'cc'})) {
+                foreach (split(/,/, $fields{'cc'})) {
+                    s/^\s+//;
+                    s/\s+$//;
+                    $cc_arg .= ' -c "' . $_ . '" ';
+                };
+            };
 	    my $subject_arg = (defined($fields{"subject"}) ? "-s '" . $fields{"subject"} . "' " : "");
 	    print "Mail $subject_arg $cc_arg '" . $fields{"to"} . "' <<'$end_of_form_marker'\n$result_body\n$end_of_form_marker\n\n";
 	} elsif ($self->{_mechanism} eq 'mh') {
