@@ -2,7 +2,7 @@
 
 #
 # dbmapreduce.pm
-# Copyright (C) 1991-2018 by John Heidemann <johnh@isi.edu>
+# Copyright (C) 1991-2024 by John Heidemann <johnh@isi.edu>
 #
 # This program is distributed under terms of the GNU general
 # public license, version 2.  See the file COPYING
@@ -211,7 +211,10 @@ is the same as the example
     cat DATA/stats.fsdb | \
 	dbmapreduce -k experiment -- dbcolstats duration
 
-except that with C<-C> there is no forking and so things run faster.
+except that with C<-C> there is much less per-key initialization,
+so it should run faster.  (When the computation is small, like
+dbrowdiff with less than 10 rows per key, startup overhead
+can make things 10x slower.)
 
 If C<dbmapreduce> is invoked from within Perl, then one can use
 a code SUB as well:
@@ -599,6 +602,7 @@ sub setup($) {
 	my $sort_column = defined($self->{_key_column}) ? $self->{_key_column} : '0';
 	my(@sort_args) = ('--nolog');
 	push(@sort_args, '--header' => $self->{_header}) if (defined($self->{_header}));
+	push(@sort_args, '--tmpdir' => $self->{_tmpdir}) if (defined($self->{_tmpdir}));
 	push(@sort_args, $sort_column);
         my($new_reader, $new_fred) = dbpipeline_filter(
 		$self->{_input},
@@ -968,7 +972,7 @@ sub finish($) {
 
 =head1 AUTHOR and COPYRIGHT
 
-Copyright (C) 1991-2018 by John Heidemann <johnh@isi.edu>
+Copyright (C) 1991-2024 by John Heidemann <johnh@isi.edu>
 
 This program is distributed under terms of the GNU general
 public license, version 2.  See the file COPYING
