@@ -2,7 +2,7 @@
 
 #
 # dbmerge.pm
-# Copyright (C) 1991-2020 by John Heidemann <johnh@isi.edu>
+# Copyright (C) 1991-2024 by John Heidemann <johnh@isi.edu>
 #
 # This program is distributed under terms of the GNU general
 # public license, version 2.  See the file COPYING
@@ -46,7 +46,7 @@ input files.
 Because this program is intended to merge multiple sources,
 it does I<not> default to reading from standard input.
 If you wish to read standard input,
-giv F<-> as an explicit input source.
+use F<-> as the input source.
 
 Also, because we deal with multiple input files,
 this module doesn't output anything until it's run.
@@ -70,10 +70,14 @@ General option:
 
 =item B<--xargs>
 
-Expect that input filenames are given, one-per-line, on standard input.
-(In this case, merging can start incrementally.)
+Expect that input filenames are given, one-per-line, on standard input
+(or from what is specified by C<--input>).
+No header is required on the input,
+but lines begining with a hash are treated as comments.
 
-=item B<--removeinputs>
+With xargs, merging will start as soon as two filenames have been given.
+
+=item B<--remove-inputs>  (also B<--removeinputs>)
 
 Delete the source files after they have been consumed.
 (Defaults off, leaving the inputs in place.)
@@ -303,7 +307,7 @@ sub parse_options($@) {
 	'j|parallelism=i' => \$self->{_max_parallelism},
 	'log!' => \$self->{_logprog},
 	'o|output=s' => sub { $self->parse_io_option('output', @_); },
-	'removeinputs!' => \$self->{_remove_inputs},
+	'removeinputs|remove-inputs!' => \$self->{_remove_inputs},
 	'test=s' => \$self->{_test},
 	'T|tmpdir|tempdir=s' => \$self->{_tmpdir},
 	'xargs!' => \$self->{_xargs},
@@ -359,7 +363,7 @@ sub segment_next_output($$) {
 	print "# final output\n" if ($self->{_debug});
     } elsif ($output_type eq 'file') {
 	# dump to a file for merging
-	my $tmpfile = Fsdb::Support::NamedTmpfile::alloc($self->{_tmpdir});
+	my $tmpfile = Fsdb::Support::NamedTmpfile::alloc($self->{_tmpdir}, 1);
 	$self->{_files_cleanup}{$tmpfile} = 'NamedTmpfile';
 	$out = $tmpfile;   # just return the name
     } elsif ($output_type eq 'ipc') {
@@ -410,8 +414,8 @@ sub segment_cleanup($$) {
 	print "# dbmerge: segment_cleanup:  no cleanup for " . _pretty_fn($file) . "\n" if ($self->{_debug});
 	# nothing
     } elsif ($cleanup_type eq 'unlink') {
-	print "# dbmerge: segment_cleanup: cleaning up $file\n" if ($self->{_debug});
-	unlink($file);
+        my ($unlink_result) = unlink($file) ? "" : " (unlink failed: $!)";
+	print "# dbmerge: segment_cleanup: cleaning up $file$unlink_result\n" if ($self->{_debug});
     } elsif ($cleanup_type eq 'NamedTmpfile') {
 	print "# dbmerge: segment_cleanup:  NamedTmpfile::cleanup_one $file\n" if ($self->{_debug});
 	Fsdb::Support::NamedTmpfile::cleanup_one($file);
@@ -663,7 +667,7 @@ Internal: read new filenames to process (from stdin)
 and send them to the work queue.
 
 Making a separate Fred to handle xargs is a lot of work, 
-but it guarantees it comes in on an IO::Handle that is selectable.
+but it guarantees each comes in on an IO::Handle that is selectable.
 
 =cut
 
@@ -735,12 +739,17 @@ Items in the work queue may not be ready.  For in-progress items we
 wait until they are done.  For not-yet-started items
 we start them, then wait until they are done.
 
-Endgame parallelism handles the final stages of a large merge.
-When there are enough processors that we can start a merge jobs
+Endgame parallelism handles the final stages of a large merge,
+when there are enough processors that we can start a merge jobs
 for all remaining levels of the merge tree.  At this point we switch
 from merging to files to merging into C<Fsdb::BoundedQueue> pipelines
 that connect merge processes which start and run concurrently.
 
+(In endgame parallelism we try to keep all processors busy.
+The CPU load of each job is unequal, since processors low in the tree
+stall on the root.  We therefore start more merge workers that we have
+processors, since many will run at only a fraction of the CPU.)
+
 The final merge is done in the main thread so that that the main thread
 can handle the output stream and recording the merge action.
 
@@ -784,8 +793,9 @@ sub segments_merge_all($) {
                     $self->{_xargs_fred}->join();
                     $self->{_xargs_fred} = undef;
                 };
+                # It's not completely clear why overall_progress == 1 when no input, but that's what happens.
                 croak($self->{_prog} . ": xargs, but no files for input\n")
-                    if ($self->{_xargs});
+                    if ($self->{_xargs} && $overall_progress <= 1);
             };
             # all done for real
             last;
@@ -857,7 +867,7 @@ sub segments_merge_all($) {
 	# because we need to alternate reaping finished processes
 	# and xargs.
 	#
-	# Fortunately, this terminates when xargs are complete.
+	# Fortunately, busy-looping terminates when xargs are complete.
        	#
 	if ($self->{_xargs_ipc_status}) {
 	    my(@ready) = $xargs_select->can_read($progress_backoff);
@@ -1046,7 +1056,7 @@ sub run($) {
 
 =head1 AUTHOR and COPYRIGHT
 
-Copyright (C) 1991-2020 by John Heidemann <johnh@isi.edu>
+Copyright (C) 1991-2024 by John Heidemann <johnh@isi.edu>
 
 This program is distributed under terms of the GNU general
 public license, version 2.  See the file COPYING
