@@ -2,7 +2,7 @@
 
 #
 # dbmultistats.pm
-# Copyright (C) 1991-2019 by John Heidemann <johnh@isi.edu>
+# Copyright (C) 1991-2024 by John Heidemann <johnh@isi.edu>
 #
 # This program is distributed under terms of the GNU general
 # public license, version 2.  See the file COPYING
@@ -248,9 +248,11 @@ sub set_defaults ($) {
     $self->{_median} = undef;   # special case: renames the output field
     $self->{_max_parallelism} = undef;
     $self->{_include_non_numeric} = undef;
+    $self->set_default_tmpdir;
     $self->{_header} = undef;
     $self->{_output_on_no_input} = undef;
     $self->{_fscode} = undef;
+    $self->{_weight_column} = undef;
 }
 
 =head2 parse_options
@@ -287,6 +289,7 @@ sub parse_options ($@) {
 	'q|quantile=i' => \$self->{_quantile},
 	'S|pre-sorted+' => \$self->{_pre_sorted},
 	'T|tmpdir|tempdir=s' => \$self->{_tmpdir},
+	'w|weight=s' => \$self->{_weight_column},
 	) or pod2usage(2);
     $self->parse_target_column(\@argv);
 }
@@ -322,6 +325,8 @@ sub setup ($) {
 	if (defined($self->{_median}));
     push(@dbcolstats_argv, '--quantile', $self->{_quantile})
 	if (defined($self->{_quantile}));
+    push(@dbcolstats_argv, '--weight', $self->{_weight_column})
+	if (defined($self->{_weight_column}));
     push(@dbcolstats_argv, '--tmpdir', $self->{_tmpdir})
 	if (defined($self->{_tmpdir}));
     push(@dbcolstats_argv, '--parallelism', $self->{_max_parallelism})
@@ -425,7 +430,7 @@ sub finish ($) {
 
 =head1 AUTHOR and COPYRIGHT
 
-Copyright (C) 1991-2015 by John Heidemann <johnh@isi.edu>
+Copyright (C) 1991-2024 by John Heidemann <johnh@isi.edu>
 
 This program is distributed under terms of the GNU general
 public license, version 2.  See the file COPYING
