@@ -2,7 +2,7 @@
 
 #
 # dbrowdiff.pm
-# Copyright (C) 1991-2022 by John Heidemann <johnh@isi.edu>
+# Copyright (C) 1991-2024 by John Heidemann <johnh@isi.edu>
 #
 # This program is distributed under terms of the GNU general
 # public license, version 2.  See the file COPYING
@@ -17,7 +17,7 @@ dbrowdiff - compute row-by-row differences of some column
 
 =head1 SYNOPSIS
 
-dbrowdiff [-B|-I] column
+dbrowdiff [-B|-I|-F] [-A AbsDiffColumnName] [-P PctDiffColumnName] column
 
 =head1 DESCRIPTION
 
@@ -28,6 +28,12 @@ C<absdiff> and C<pctdiff>.
 Differences are either relative to the previous column 
 (I<incremental> mode), or relative to the first row
 (I<baseline> mode), the default.
+Alternatively, in I<future> mode, differences
+are between the I<next> row and the current row.
+
+If column names are given, with C<-A> or C<-P>,
+then only columns with that name are produced.
+
 
 =head1 OPTIONS
 
@@ -41,11 +47,30 @@ Select baseline mode (the default), where differences are relative to the first
 
 Select incremental mode, where differences are relative to the previous row.
 
+=item B<-F> or B<--future>
+
+Select future  incremental mode, where differences are
+incremental between the next row and the current one.
+
+=item B<-A> COL or B<--absdiff> COL
+
+Name the absolute difference output column COL,
+and don't output percent difference unless C<-P> is given.
+
+=item B<-P> COL or B<--pctdiff> COL
+
+Name the percent difference output column COL,
+and don't output absolute difference unless C<-P> is given.
+
 =item B<-f FORMAT> or B<--format FORMAT>
 
 Specify a L<printf(3)>-style format for output statistics.
 Defaults to C<%.5g>.
 
+=item B<-e> EmptyValue or B<--empty>
+
+Specify the value for the last row when in future mode.
+
 =back
 
 
@@ -184,6 +209,8 @@ sub set_defaults ($) {
     $self->SUPER::set_defaults();
     $self->{_format} = "%.5g";
     $self->{_mode} = 'B';
+    $self->{_absdiff_column} = undef;
+    $self->{_pctdiff_column} = undef;
 }
 
 =head2 parse_options
@@ -203,14 +230,18 @@ sub parse_options ($@) {
  	'help|?' => sub { pod2usage(1); },
 	'man' => sub { pod2usage(-verbose => 2); },
 	'autorun!' => \$self->{_autorun},
+        'A|absdiff=s' => \$self->{_absdiff_column},
 	'B|baseline' => sub { $self->{_mode} = 'B'; },
 	'close!' => \$self->{_close},
 	'd|debug+' => \$self->{_debug},
+	'e|empty=s' => \$self->{_empty},
 	'f|format=s' => \$self->{_format},
+	'F|future' => sub { $self->{_mode} = 'F'; },
 	'i|input=s' => sub { $self->parse_io_option('input', @_); },
 	'I|incremental' => sub { $self->{_mode} = 'I'; },
 	'log!' => \$self->{_logprog},
 	'o|output=s' => sub { $self->parse_io_option('output', @_); },
+        'P|pctdiff=s' => \$self->{_pctdiff_column},
 	) or pod2usage(2);
     $self->parse_target_column(\@argv);
 }
@@ -228,16 +259,33 @@ sub setup ($) {
 
     pod2usage(2) if (!defined($self->{_target_column}));
 
-    $self->finish_io_option('input', -comment_handler => $self->create_pass_comments_sub);
+    $self->finish_io_option('input', -comment_handler =>
+                            ($self->{_mode} eq "F" ?
+                             $self->create_delay_comments_sub :
+                             $self->create_pass_comments_sub));
 
     $self->{_target_coli} = $self->{_in}->col_to_i($self->{_target_column});
     croak($self->{_prog} . ": target column " . $self->{_target_column} . " is not in input stream.\n")
 	if (!defined($self->{_target_coli}));
 
+    my(@new_colnames);
+    if (defined($self->{_absdiff_column} && defined($self->{_pctdiff_column}))) {
+        @new_colnames = ($self->{_absdiff_column}, $self->{_pctdiff_column});
+    } elsif (defined($self->{_absdiff_column})) {
+        @new_colnames = ($self->{_absdiff_column}, undef);
+    } elsif (defined($self->{_pctdiff_column})) {
+        @new_colnames = (undef, $self->{_pctdiff_column});
+    } else {
+        $self->{_absdiff_column} = 'absdiff';
+        $self->{_pctdiff_column} = 'pctdiff';
+        @new_colnames =  qw(absdiff pctdiff);
+    };
+            
     $self->finish_io_option('output', -clone => $self->{_in}, -outputheader => 'delay');
     my($destination_type) = ($self->{_in}->col_type_is_numeric($self->{_target_column}) == 1 ? 'q' : 'd');
-    foreach ("absdiff:$destination_type", "pctdiff:$destination_type") {
-	$self->{_out}->col_create($_)
+    foreach (@new_colnames) {
+        next if (!defined($_));
+	$self->{_out}->col_create("$_:$destination_type")
 	    or croak($self->{_prog} . ": cannot create column $_ (maybe it already existed?)\n");
     };
 }
@@ -256,38 +304,78 @@ sub run ($) {
     my $write_fastpath_sub = $self->{_out}->fastpath_sub();
 
     my $target_coli = $self->{_target_coli};
-    my $absdiff_coli = $self->{_out}->col_to_i('absdiff');
-    my $pctdiff_coli = $self->{_out}->col_to_i('pctdiff');
+    my $absdiff_coli = defined($self->{_absdiff_column}) ? $self->{_out}->col_to_i($self->{_absdiff_column}) : undef;
+    my $pctdiff_coli = defined($self->{_pctdiff_column}) ? $self->{_out}->col_to_i($self->{_pctdiff_column}) : undef;
     my $format = $self->{_format};
     my $incremental_mode = ($self->{_mode} eq 'I');
+    my $future_mode = ($self->{_mode} eq 'F');
 
     my $base;
     my $absdiff;
     my $pctdiff;
-    my $fref;
+    my $fref = undef;
+    my $next_fref;
 
-    while ($fref = &$read_fastpath_sub()) {
-	if (!defined($base)) {
-	    $absdiff = $pctdiff = 0.0;
-	    $base = $fref->[$target_coli];
-	} else {
-	    $absdiff = $fref->[$target_coli] - $base;
-	    $pctdiff = ($absdiff / $base) * 100.0 if ($base != 0);
-	};
-	$fref->[$absdiff_coli] = sprintf("$format", $absdiff);
-	if ($base == 0) {
-	    $fref->[$pctdiff_coli] = $self->{_empty};
-	} else {
-	    $fref->[$pctdiff_coli] = sprintf("$format", $pctdiff);
-	};
-	$base = $fref->[$target_coli] if ($incremental_mode);
-	&$write_fastpath_sub($fref);
+    if ($future_mode) {
+        my($last_fref) = undef;
+        my $input_delay_comments = $self->{_delay_comments}[0];
+        while ($fref = &$read_fastpath_sub()) {
+            if (!defined($last_fref)) {
+                $last_fref = $fref;
+                next;
+            };
+            $absdiff = $fref->[$target_coli] - $last_fref->[$target_coli];
+            $pctdiff = ($absdiff / $last_fref->[$target_coli]) * 100.0;
+            # emit one behind
+            if (defined($absdiff_coli)) {
+                $last_fref->[$absdiff_coli] = sprintf("$format", $absdiff);
+            };
+            if (defined($pctdiff_coli)) {
+                $last_fref->[$pctdiff_coli] = sprintf("$format", $pctdiff);
+            };
+            &$write_fastpath_sub($last_fref);
+            $input_delay_comments->flush($self->{_out});
+            $last_fref = $fref;
+        };
+        # output saved row, if any
+        if (defined($last_fref)) {
+            if (defined($absdiff_coli)) {
+                $last_fref->[$absdiff_coli] = $self->{_empty};
+            };
+            if (defined($pctdiff_coli)) {
+                $last_fref->[$pctdiff_coli] = $self->{_empty};
+            };
+            &$write_fastpath_sub($last_fref)
+        };
+    } else {
+        # non-future mode
+        while ($fref = &$read_fastpath_sub()) {
+            if (!defined($base)) {
+                $absdiff = $pctdiff = 0.0;
+                $base = $fref->[$target_coli];
+            } else {
+                $absdiff = $fref->[$target_coli] - $base;
+                $pctdiff = ($absdiff / $base) * 100.0 if ($base != 0);
+            };
+            if (defined($absdiff_coli)) {
+                $fref->[$absdiff_coli] = sprintf("$format", $absdiff);
+            };
+            if (defined($pctdiff_coli)) {
+                if ($base == 0) {
+                    $fref->[$pctdiff_coli] = $self->{_empty};
+                } else {
+                    $fref->[$pctdiff_coli] = sprintf("$format", $pctdiff);
+                };
+            };
+            $base = $fref->[$target_coli] if ($incremental_mode);
+            &$write_fastpath_sub($fref);
+        };
     };
 }
 
 =head1 AUTHOR and COPYRIGHT
 
-Copyright (C) 1991-2022 by John Heidemann <johnh@isi.edu>
+Copyright (C) 1991-2024 by John Heidemann <johnh@isi.edu>
 
 This program is distributed under terms of the GNU general
 public license, version 2.  See the file COPYING
