@@ -72,6 +72,14 @@ sort in reverse order (high to low)
 
 sort in normal order (low to high)
 
+=item B<-t> or B<--type-inferred-sorting>
+
+sort fields by type (numeric or leicographic), automatically
+
+=item B<-T> or B<--no-type-inferred-sorting>
+
+sort fields only as specified based on C<-n> or C<-N>
+
 =item B<-n> or B<--numeric>
 
 sort numerically
@@ -248,6 +256,7 @@ sub parse_options ($@) {
 	'N|lexical' => sub { $self->parse_sort_option(@_); },
 	'r|descending' => sub { $self->parse_sort_option(@_); },
 	'R|ascending' => sub { $self->parse_sort_option(@_); },
+	't|type-inferred-sorting' => sub { $self->parse_sort_option(@_); },
 	'<>' => sub { $self->parse_sort_option('<>', @_); },
 	) or pod2usage(2);
     croak($self->{_prog} . ": internal error, extra arguments.\n")
@@ -336,7 +345,7 @@ sub segment_next_output($$) {
 	print "# dbsort segment_next_output: final output\n" if ($self->{_debug});
     } else {
 	# dump to a file for merging
-	my $tmpfile = Fsdb::Support::NamedTmpfile::alloc($self->{_tmpdir});
+	my $tmpfile = Fsdb::Support::NamedTmpfile::alloc($self->{_tmpdir}, 1);
 	$out = $tmpfile;   # just return the name
 	push(@{$self->{_files_to_merge}}, $tmpfile);
 	print "# dbsort segment_next_output: intermediate file: $tmpfile\n" if ($self->{_debug});
@@ -410,8 +419,12 @@ sub segment_merge_start($$) {
 	my(@merge_args) = qw(--nolog --noclose --removeinputs --xargs);
 	push(@merge_args, '--parallelism', $self->{_max_parallelism})
 	    if (defined($self->{_max_parallelism}));
+	push(@merge_args, '--header', $self->{_header})
+	    if (defined($self->{_header}));
 	push(@merge_args, '-T', $self->{_tmpdir})
 	    if (defined($self->{_tmpdir}));
+        push(@merge_args, '-d')
+            if ($self->{_debug});
 	push(@merge_args, @{$self->{_sort_argv}});
 
 	my($writer, $merge_fred) = dbpipeline_sink(\@writer_args,
