@@ -100,6 +100,11 @@ or a more modern form
 
     17:00:14.808855 IP 10.0.0.172.31738 > 10.1.0.50.telnet: Flags [S], seq 3236187954, win 21463, length 0
 
+and
+
+    15:29:11.162365 IP6 2001:1878:404:f200::76d.41392 > 2001:1878:401::8009:1c09.https: Flags [S], seq 485329338, win 64660, options [mss 1220,sackOK,TS val 3349932199 ecr 0,nop,wscale 10], length 0
+
+
 =head2 Command:
 
     tcpdump_to_db
@@ -210,7 +215,7 @@ sub setup ($) {
 
     $self->finish_fh_io_option('input');
 
-    $self->finish_io_option('output', -cols => [qw(time proto src dest flags start end len ack win)]);
+    $self->finish_io_option('output', -cols => [qw(time proto src dest flags start end len ack win seq)]);
 }
 
 =head2 _conv_time
@@ -264,34 +269,76 @@ sub run ($) {
 
         next if ($line !~ /^\d/);
         my(@F) = split(' ', $line);
-        my($proto, $raw_time, $src, $dummy, $dest, $flags, $ack, $win, $start, $end, $len) = ($empty) x 20;
+        my($ip_version, $proto, $raw_time, $src, $dummy, $dest, $flags, $ack, $win, $start, $end, $len, $seq, $options) = ($empty) x 20;
         $raw_time = shift @F;
         # xxx: should support other protos!
         # Currently silently fails on them.
+        # With "modern from", the second field is protocol (IP or IP6)
+        if ($F[0] eq 'IP' || $F[0] eq 'IP6') {
+            $ip_version = shift @F;
+        } else {
+            $ip_version = 'IP';
+        };
         $proto = 'tcp';
-        $src = shift @F;
+        #
         # The src field may have "truncated-ip" instead of the src.
         # If the entry is truncated then silently ignore it.
+        $src = shift @F;
         next if ($src =~ /truncated/);
     
-        $dummy = shift @F;
+        $dummy = shift @F;  # >
         $dest = shift @F;
         $dest =~ s/:$//;
-        $flags = shift @F;
-        if ($F[0] =~ /^\d/) {
-	    ($start, $end, $len) = ($F[0] =~ /^\s*(\d+):(\d+)\((\d+)\)\s*$/);
+        if ($F[0] eq 'Flags') {
+            shift @F;
+            $flags = shift @F;
+            $flags =~ s/^\[//;
+            $flags =~ s/\]?,$//;
+        } else {
+            $flags = shift @F;
+        };
+        if ($F[0] =~ /^\d+:\d+\(\d+\)/) {
+            ($start, $end, $len) = ($F[0] =~ /^\s*(\d+):(\d+)\((\d+)\)\s*$/);
+            shift @F;
+        };
+        if ($F[0] eq 'seq') {
 	    shift @F;
+	    $seq = shift @F;
+            $seq =~ s/,$//;
         };
         if ($F[0] eq 'ack') {
 	    shift @F;
 	    $ack = shift @F;
+            $ack =~ s/,$//;
         };
         if ($F[0] eq 'win') {
 	    shift @F;
 	    $win = shift @F;
+            $win =~ s/,$//;
+        };
+        if ($#F >= 0 && $F[0] eq 'options') {
+	    shift @F;
+            $options = '';
+            my $sep = '';
+            $F[0] =~ s/^\[//;
+            my $done = undef;
+            while (!$done && $#F >= 0) {
+                if ($F[0] =~ /\],$/) {
+                    $done = 1;
+                    $F[0] =~ s/\],$//;
+                    $sep = ':';
+                };
+                $options .= $sep . shift @F;
+                $sep = ':';
+            };
+        };
+        if ($#F >= 0 && $F[0] eq 'length') {
+	    shift @F;
+	    $len = shift @F;
+            $len =~ s/,$//;
         };
 
-	my @outf = ($self->_conv_time($raw_time), $proto, $src, $dest, $flags, $start, $end, $len, $ack, $win);
+	my @outf = ($self->_conv_time($raw_time), $proto, $src, $dest, $flags, $start, $end, $len, $ack, $win, $seq);
 	&{$write_fastpath_sub}(\@outf);
     };
 }
