@@ -2,7 +2,7 @@
 
 #
 # Fsdb::Support::IPv6.pm
-# Copyright (C) 2021 by John Heidemann <johnh@ficus.cs.ucla.edu>
+# Copyright (C) 2021-2025 by John Heidemann <johnh@isi.edu>
 #
 # This program is distributed under terms of the GNU general
 # public license, version 2.  See the file COPYING
@@ -31,6 +31,8 @@ Fsdb::Support::IPv6 - ipv6-parsing helpers
 Normalize an IPv6 address so it has all hextets present,
 with no leading zeros.
 
+Handles any valid IPv6 address as input.
+
 Also treats strings of hex digits without colons as IPv6 prefixes.
 
 =cut
@@ -47,6 +49,10 @@ sub ipv6_zeroize {
         $s =~ s/(....)/$1:/g;
         $s =~ s/:$//;
     }
+    # handle embeded IPv4
+    if ($s =~ /^(.*:)(\d+)\.(\d+)\.(\d+)\.(\d+)$/) {
+        $s = $1 . sprintf("%02x%02x:%02x%02x", $2, $3, $4, $5);
+    };
 #    if ($s !~ /::/) {
 #        return $s;
 #    };
@@ -89,7 +95,7 @@ sub ipv6_zeroize {
 
 =head2 ipv6_fullhex
 
-    $ipv6_fullhex = Fsdb::Support::IPv6::ipv6_full('1:0002:3:4::');
+    $ipv6_fullhex = Fsdb::Support::IPv6::ipv6_fullhex('1:0002:3:4::');
     # result is 0001000200030004000000000000000
 
 Rewrite an IPv6 address as a full, 128-bit, base-16 number.
@@ -114,6 +120,8 @@ or :: in a non-standard place.
 Normalize it to remove leading zeros and replace the leftmost,
 longest run of zeros with ::.
 
+We remove any mixed IPv4/v6 addresses (IPv6 addreses that end in dotted-quads).
+
 Also treats strings of hex digits without colons as IPv6 prefixes.
 
 =cut
@@ -302,7 +310,7 @@ sub _test_ipv6 {
     # _test_zero_remove("1:2::6:0:0:8",    "1:2:0:0:6:0:0:8");
     _test_zero_fill("1:002::6:0:0:8",    "1:2:0:0:6:0:0:8");
     _test_zero_fill("1:2:0:0:6::8",    "1:2:0:0:6:0:0:8");  # technically non-compliant
-    _test_zero_fill("1:2::6::8",       undef);              # ambiguous, coudl be 1:2:0:0:6:0:0:8, or 1:2:0:0:0:6:0:8, or other configs
+    _test_zero_fill("1:2::6::8",       undef);              # ambiguous, could be 1:2:0:0:6:0:0:8, or 1:2:0:0:0:6:0:8, or other configs
     _test_fullhex("1:2:3:4:5:6:7:8", "00010002000300040005000600070008");
     _test_fullhex("1:002::6:0:0:8",  "00010002000000000006000000000008");
     _test_fullhex("1:abcd::6:0:0:8", "0001abcd000000000006000000000008");
@@ -313,6 +321,8 @@ sub _test_ipv6 {
     _test_zero_fill("000100020003", "1:2:3:0:0:0:0:0");
     _test_fullhex_to_normal('20010db8000300040005000600070008', '2001:db8:3:4:5:6:7:8');
     _test_fullhex_to_normal('c0000201', '192.0.2.1');
+    # new case 2025-03-01: ipv4-mapped IPv6
+    _test_zero_fill("::ffff:192.0.2.128",    "0:0:0:0:0:ffff:c000:280");
 };
 
 # _test_ipv6;
