@@ -3,7 +3,7 @@
 #
 # Fsdb::Support::NamedTmpfile.pm
 # Copyright (C) 2007 by John Heidemann <johnh@isi.edu>
-# $Id$
+# $Id: b84f24f02848d9777818453f2ced50674f8caa28 $
 #
 # This program is distributed under terms of the GNU general
 # public license, version 2.  See the file COPYING
@@ -61,8 +61,28 @@ my %named_tmpfiles;
 my $tmpdir = undef;
 my $template = undef;
 
-sub alloc {
-    my($tmpdir) = @_;
+my $have_snzip = undef;
+
+=head2 cleanup_one
+
+    Fsdb::Support::NamedTmpfile::alloc($directory, $compression);
+
+Create a new tmpfile in DIRECTORY, optionally with COMPRESSION.
+
+
+=cut
+
+sub alloc(;$$) {
+    my($tmpdir, $compression) = @_;
+
+    my($suffix) = "~";
+    if ($compression) {
+        if (!defined($have_snzip)) {
+            $have_snzip = (-x "/usr/bin/snzip" ? 1 : 0);
+        };
+        # Snzip *requires* a .sz extension or snzip throws an error.
+        $suffix = $have_snzip ? ".sz" : ".gz~";
+   };
 
     if (!defined($tmpdir)) {
 	$tmpdir = (defined($ENV{'TMPDIR'}) ? $ENV{'TMPDIR'} : "/tmp") if (!defined($tmpdir));
@@ -73,10 +93,13 @@ sub alloc {
 
     croak "tmpdir $tmpdir is not a directory\n" if (! -d $tmpdir);
     croak "tmpdir $tmpdir is not writable\n" if (! -w $tmpdir);
-    my($fh, $fn) = tempfile($template, SUFFIX => "~", DIR => $tmpdir);
+    my($fh, $fn) = tempfile($template, SUFFIX => $suffix, DIR => $tmpdir);
     close $fh;
     {
-#	lock($named_tmpfiles_lock);
+        #	lock($named_tmpfiles_lock);
+        # We save the pid of the creator, so only that pid will clean it up.
+        # If we don't do this and fork, then each child will also attempt to cleanup
+        # and tmpfiles will get prematurely reaped.
 	$named_tmpfiles{$fn} = $$;
     }
 
